import tornado.tcpserver
from tornado.gen import *
from tornado.concurrent import *
import tornado.iostream
import tornado.ioloop
import itertools
import json
import traceback
import time
import shmooze.lib.packet as packet
from toro import *
import datetime
import socket

ioloop=tornado.ioloop.IOLoop.instance()

@coroutine
def accept(sock):
    @return_future
    def wrap_add_handler(callback):
       def wait_for_accept(fd,events): # TODO more intelligently check for events?
            try:
                result = sock.accept()
                callback(result)
            except socket.error, e:
                if e.args[0] not in (errno.EWOULDBLOCK, errno.EAGAIN):
                    raise

       ioloop.add_handler(sock.fileno(), wait_for_accept, ioloop.READ)

    sock.setblocking(0)
    try:
        result = yield wrap_add_handler()
        raise Return(result)
    finally:
        ioloop.remove_handler(sock.fileno())

@coroutine
def wait(proc):
    poll_period=datetime.timedelta(milliseconds=10)
    while True:
        p=proc.poll()
        if p is not None:
            raise Return(p)

        @return_future
        def pause(callback):
            ioloop.add_timeout(poll_period,callback)

        yield pause()

def connection_ready(sock, fd, events):
    while True:
        try:
            connection, address = sock.accept()
        except socket.error, e:
            if e.args[0] not in (errno.EWOULDBLOCK, errno.EAGAIN):
                raise
            return
        connection.setblocking(0)
        handle_connection(connection, address)

@coroutine
def listen_for_commands(stream,handle_cr,over_fn=None):
    try:
        while True:
            data = yield stream.read_until('\n')
            parsed = json.loads(data)
            response = yield handle_cr(parsed)
            encoded = json.dumps(response)+'\n'
            yield stream.write(encoded)
    except tornado.iostream.StreamClosedError:
        pass
    except Exception:
        print "Communication exception!"
        traceback.print_exc()
        print "(communication interrupted)"
    finally:
        stream.close()
    if over_fn:
        over_fn()

@coroutine
def json_query(addr,port,inp,timeout=2):
    @coroutine
    def talk(stream):
        yield stream.connect((addr,port))
        encoded = json.dumps(inp)+'\n'
        yield stream.write(encoded)
        data = yield stream.read_until('\n')
        decoded = json.loads(data)
        raise Return(decoded)

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    stream = tornado.iostream.IOStream(s)
    try:
        result = yield with_timeout(datetime.timedelta(seconds=timeout),talk(stream))
    finally:
        stream.close()
    raise Return(result)

class Service(tornado.tcpserver.TCPServer):
    def __init__(self,port=None):
        if port is not None:
            self.port = port
        tornado.tcpserver.TCPServer.__init__(self)
        self.listen(self.port)

    # Override this
    @coroutine
    def command(self,query):
        raise Return(query)

    def handle_stream(self,stream,address):
        return listen_for_commands(stream,self.command)

class JSONCommandProcessor(object):
    @coroutine
    def command(self,line):
        if isinstance(line,list):
            try:
                result=[]
                for c in line:
                    single_result=yield self.single_command(c)
                    result.append(single_result)
            except Exception:
                traceback.print_exc()
                result = packet.error("Generic multi-command processing error")
            finally:
                raise Return(result)
        elif isinstance(line,dict):
            try:
                result = yield self.single_command(line)
            except Exception:
                traceback.print_exc()
                result = packet.error("Generic command processing error")
            finally:
                raise Return(result)
        else:
            raise Return(packet.error("Command must be either dict (single command) or list (multiple commands)"))

    # Parse and run a command
    @coroutine
    def single_command(self,line):
        if not isinstance(line,dict):
            raise Return(packet.error('Command not a dict.'))

        try:
            cmd=line['cmd'] # Fails if no cmd given
        except KeyError:
            raise Return(packet.error('No command given.'))

        try:
            args=line['args']
        except KeyError:
            args={}

        if not isinstance(args,dict):
            raise Return(packet.error('Argument list not a dict.'))

        try:
            f=self.commands[cmd]
        except KeyError:
            raise Return(packet.error('Bad command.'))

        try:
            result=yield f(self,**args)
            result=packet.good(result)
        except Exception as e:
            traceback.print_exc()
            result=packet.error(str(e))

        if cmd in self.log_cmds and self.logger:
            #self.logger.log({'timestamp':str(datetime.datetime.utcnow()),'id':self.log_prefix,'sent':line,'received':result})
            self.logger.log(self.log_uid, self.log_namespace, line, result)

        raise Return(result)

    commands = {}
    log_cmds = []
    log_uid = None
    log_namespace = None
    logger = None



import shmooze.lib.cmdlog
import shmooze.lib.database as database
import shmooze.lib.service as service
import shmooze.settings as settings
import uuid

# A pool manages the life and death of modules, through tornado's IOLoop.

class Pool(service.JSONCommandProcessor, service.Service):
    port=settings.ports["pool"]

    def __init__(self,modules,logfilename=None):
        print "Pool started."
        # Create a UUID for this instance
        self.instance = str(uuid.uuid4())

        # Create lookup table of possible modules & backgrounds
        self.modules_available_dict = dict([(m.TYPE_STRING,m) for m in modules])

        # pool is the actual pool of modules
        self.pool=set()
        # pool is a synchronization object so that multiple clients don't try to alter the pool at the same time
        # (also includes background)
        self.pool_lock=service.Lock()

        # old_queue is used to take diffs of the pool (and from there, send appropriate messages to affected modules.)
        # whenever the pool is unlocked, it should equal the pool.
        self.old_pool=[]

        # When debugging, uids are assigned sequentially
        self.debug = False

        # Each module on the pool gets a unique ID, this variable allocates those
        self.last_uid=-1

        # Log important commands
        # (used in JSONCommandProcessor)
        if logfilename:
            self.logger = database.Database(log_table="pool_log")
        self.log_namespace = "client-pool"

        # JSONCommandService handles all of the low-level TCP connection stuff.
        super(Pool,self).__init__()

    # Get a new UID for a module.
    def get_uid(self):
        if self.debug:
            self.last_uid += 1
            return self.last_uid
        return str(uuid.uuid4())

    # Called from client
    # Retrieves given parameters from the module
    @service.coroutine
    def ask_module(self,uid,parameters):
        d=dict(self.pool)
        if uid not in d:
            raise Exception("Module identifier not in queue")
        raise service.Return(d[uid].get_multiple_parameters(parameters))

    # Called from client
    # Retrieves names of possible modules that can be added to the pool
    @service.coroutine
    def modules_available(self):
        raise service.Return(self.modules_available_dict.keys())

    # Called from client
    # Retrieves the current pool, and info about modules on it
    @service.coroutine
    def get_pool(self,parameters={}):
        l=[]
        for (uid,obj) in self.pool:
            d={'uid':uid,'type':obj.TYPE_STRING}
            if obj.TYPE_STRING in parameters:
                d['parameters']=obj.get_multiple_parameters(parameters[obj.TYPE_STRING])
            l.append(d)
        raise service.Return(l)

    # Called from client
    # Issues a command to a module
    # Note that this involves a transaction between the pool and the module, and may take a while.
    # This is in contrast to ask_module which only retrieves cached information and does not create additional transactions.
    @service.coroutine
    def tell_module(self,uid,cmd,args={}):
        d=dict(self.pool)
        if uid not in d:
            raise Exception("Module identifier not in pool")
        result = yield d[uid].tell(cmd,args)
        raise service.Return(result)

    # Called from client
    # Create a new module and add it to the pool
    # May take a little while as module is spawned and constructed.
    @service.coroutine
    def add(self,type,args={}):
        uid=self.get_uid()
        if type not in self.modules_available_dict:
            raise Exception("Unrecognized module name")
        mod_inst=self.modules_available_dict[type](self.get_remover(uid))
        mod_inst.logger = self.logger
        mod_inst.uid = uid 
        mod_inst.log_uid = uid 
        mod_inst.log_namespace = "module-instance" 
        yield mod_inst.new(args)
        with (yield self.pool_lock.acquire()):
            self.pool.append((uid,mod_inst))
            yield self.pool_updated()
        raise service.Return({'uid':uid})

    # Called from client
    # Removes some modules from the pool
    # May take a little while as the modules are destroyed.
    @service.coroutine
    def rm(self,uids):
        with (yield self.pool_lock.acquire()):
            self.pool={(uid,obj) for (uid,obj) in self.pool if uid not in uids}
            if self.bg is not None and self.bg[0] in uids:
                self.bg=None
            yield self.pool_updated()

    # Take a diff of the pool, and issue appropriate commands to modules (start and rm) if necessary.
    # May take a little while as the commands are executed.
    # Commands are executed simultaneously.
    # Pool should be locked for this operation
    # TODO harden this
    @service.coroutine
    def pool_updated(self):
        again=True
        while again:
            again=False
            cur_uids=[uid for (uid,obj) in self.pool]
            to_remove=[((uid,obj),obj.remove) for (uid,obj) in self.old_pool if uid not in cur_uids and obj.alive]
            to_play=[]
            for (uid, obj) in self.pool:
                if not obj.is_on_top: # is_on_top is alias for 'is running'  in this case
                    to_play.append(((uid, obj), obj.play))

            to_suspend = [] # Nothing should be suspended

            self.old_pool=self.pool

            actions=to_remove+to_suspend+to_play
            try:
                if len(actions) > 0:
                    # Execute all operations simultaneously
                    actions=[(mod,future()) for mod,future in actions]
                    yield [future for uid,future in actions]
            except Exception as e:
                print "Errors trying to update poool:"
                for (uid,obj),f in actions:
                    if f.exception():
                        print "- {0} raised {1}".format(uid,f.exception())
                bad_modules=[mod for mod,f in actions if f.exception()]
                print "Removing bad modules:",bad_modules
                for uid,obj in bad_modules:
                    obj.terminate()
                self.pool={(uid,obj) for uid,obj in self.pool if uid not in [uid2 for uid2,obj2 in bad_modules]}
                again=True

    # Returns a coroutine that may be executed to remove the current module from the queue
    # Generally, the result of this function is passed into a newly constructed module, so that
    # it may gracefully remove itself if it terminates naturally.
    def get_remover(self,my_uid):
        @service.coroutine
        def remove_self():
            with (yield self.pool_lock.acquire()):
                self.pool={(uid,obj) for (uid,obj) in self.pool if uid != my_uid}
                yield self.pool_updated()
        return remove_self

    def shutdown(self):
        def shutdown_complete(f):
            service.ioloop.stop()
        service.ioloop.add_future(self.killall(),shutdown_complete)

    @service.coroutine
    def killall(self):
        with (yield self.pool_lock.acquire()):
            self.pool = set()
            yield self.pool_updated()

    commands = {
        'rm':rm,
        'add':add,
        'pool':get_pool,
        'modules_available':modules_available,
        'tell_module':tell_module,
        'ask_module':ask_module,
    }

    log_cmds = ['rm','add','tell_module']



#!/usr/bin/env python2

import urllib
import urllib2
import json
import sys

DEFAULT_ENDPOINT = "http://localhost:8080/queue"

def request(message, endpoint=DEFAULT_ENDPOINT):
    req = urllib2.Request(endpoint)
    req.add_header('Content-type', 'text/json')
    req.data=json.dumps(message)

    handler = urllib2.urlopen(req)
    result=json.loads(handler.read())
    if result['success']:
        return result['result']
    else:
        raise Exception(result['error'])



import werkzeug
import json
import socket

def wsgi_control(addr,port,timeout=10):

    def query(inp):
        data=json.dumps(inp)
        s=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(timeout)
        s.connect((addr,port))
        s.sendall(data+'\n')
        result=''
        while True:
            result+=s.recv(4096)
            if '\n' in result:
                result=result[0:result.find('\n')]
                break
        s.close()
        return json.loads(result)

    @werkzeug.Request.application
    def wsgi(request):
        mime_type = request.headers.get('content-type').partition(';')[0]
        if mime_type in {'text/json', 'application/json'}:
            inp=json.loads(request.data)
            try:
                outp=query(inp)
            except Exception as e:
                return werkzeug.exceptions.InternalServerError(e)
            return werkzeug.Response(json.dumps(outp),content_type='text/json')
        return werkzeug.Response('Endpoint only accepts JSON.')
    return wsgi



import json
import sqlite3
import shmooze.settings as settings

def row_dict(r):
    # Convert a sqlite3.Row object to a dictionary
    return {k: r[k] for k in r.keys()}
        
class Database(object):
    def __init__(self, filename=None, log_table=None):
        if filename is None:
            try:
                filename = settings.log_database_path
            except:
                filename = ":memory:"

        self.conn = sqlite3.connect(filename)
        self.conn.row_factory = sqlite3.Row
        #XXX check to make sure that log_table is 'safe'
        self.log_table = log_table
        if self.log_table is not None:
            self.create_log_schema()

    def execute(self, _sql_command, **kwargs):
        return self.conn.execute(_sql_command, kwargs)

    def execute_select(self, _sql_command, **kwargs):
        return self.execute(_sql_command, **kwargs)

    def commit(self):
        return self.conn.commit()

    def log(self, uid, namespace, command, response, raw=False):
        if raw:
            input_json = command
            output_json = response
        else:
            input_json = json.dumps(command)
            output_json = json.dumps(response)
        self.execute("INSERT INTO {} (uid, namespace, input_json, output_json) VALUES (:uid, :nspace, :input_json, :output_json);".format(self.log_table),
                     uid=uid, nspace=namespace, input_json=input_json, output_json=output_json)

        self.commit()

    #def queue_log(self, action, target, raw_command=''):
    #    self.execute("INSERT INTO queue (action, target, command) VALUES (:action, :target, :command);",
    #                 action=action, target=target, command=json.dumps(raw_command))

    def create_log_schema(self):
        self.execute("""CREATE TABLE IF NOT EXISTS {} (
            pk INTEGER PRIMARY KEY,
            uid TEXT,
            namespace TEXT,
            timestamp DATETIME DEFAULT CURRENT_TIMESTAMP,
            input_json TEXT,
            output_json TEXT
        );""".format(self.log_table))
        self.commit()

    def destroy_top_schema(self):
        self.execute("DROP TABLE IF EXISTS top_module")
        self.execute("DROP TABLE IF EXISTS top_module_item")
        self.execute("DROP TABLE IF EXISTS top_category")
        self.execute("DROP TABLE IF EXISTS top_item")
        self.execute("DROP TABLE IF EXISTS top_log_entry")
        self.execute("DROP TABLE IF EXISTS top_module_log_entry")
        self.commit()

    def create_top_schema(self):
        """
        (Category --->) Item <---> Module <---> LogEntry

        <---> is a many-to-many relationship
         ---> is a foreign key relationship

        (- Category: represents a group of Items which form a top list)
         - Item: something that can be played multiple times and is grouped by to build a top list
         - Module: an instance of a module on the queue
         - LogEntry: an act performed on the queue and logged
        """
        self.execute("""CREATE TABLE IF NOT EXISTS top_module (
            uuid TEXT,
            add_timestamp DATETIME
        );""")
        self.execute("""CREATE TABLE IF NOT EXISTS top_category (
            pk INTEGER PRIMARY KEY,
            slug TEXT,
            description TEXT
        );""")
        self.execute("""CREATE TABLE IF NOT EXISTS top_item (
            pk INTEGER PRIMARY KEY,
            canonical_id TEXT,
            category_pk INTEGER,
            requeue_command TEXT,
            url TEXT,
            description TEXT
        );""")
        self.execute("""CREATE TABLE IF NOT EXISTS top_item_module (
            pk INTEGER PRIMARY KEY,
            item_pk INTEGER,
            module_uuid TEXT
        );""")
        self.execute("""CREATE TABLE IF NOT EXISTS top_log_entry (
            pk INTEGER,
            uid TEXT,
            namespace TEXT,
            timestamp DATETIME,
            input_json TEXT,
            output_json TEXT
        );""")
        self.execute("""CREATE TABLE IF NOT EXISTS top_module_log_entry (
            pk INTEGER PRIMARY KEY,
            log_pk INTEGER,
            module_uuid TEXT,
            log_type TEXT
        );""")
        self.commit()



import util
import pkg_resources
import shmooze.settings as settings

static_endpoints = {
    '/': settings.static_path
}

wsgi_endpoints = {}

for service, portnum in settings.ports.items():
    if service != "wsgi":
        wsgi_endpoints["/{}".format(service)] = util.wsgi_control("localhost", portnum)



import json
import socket
import sys
import traceback

import shmooze.lib.packet as packet

# Modules can be written in any language, but if you choose to write them in python.
# you may find the contents of this file helpful.

# Connects back to the queue based on command-line arguments
class ParentConnection(object):
    def __init__(self):
        host = sys.argv[-3]
        cmd_port = int(sys.argv[-2])
        update_port = int(sys.argv[-1])
        self.cs=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        self.us=socket.socket(socket.AF_INET,socket.SOCK_STREAM)
        self.cs.connect((host,cmd_port))
        self.us.connect((host,update_port))

        self.cs_buffer=''
        self.us_buffer=''

    # Blocks until a command has been received and returns it
    def recv_cmd(self):
        while True:
            self.cs_buffer+=self.cs.recv(4096)
            a=self.cs_buffer.find('\n')
            if a >= 0:
                cmd=self.cs_buffer[0:a]
                self.cs_buffer=self.cs_buffer[a+1:]
                break
        return json.loads(cmd)

    # Blocks until an update has been acknowledged
    def recv_update_resp(self):
        while True:
            self.us_buffer+=self.us.recv(4096)
            a=self.us_buffer.find('\n')
            if a >= 0:
                resp=self.us_buffer[0:a]
                self.us_buffer=self.us_buffer[a+1:]
                break
        resp_dict = json.loads(resp)
        packet.assert_success(resp_dict)
        return resp_dict['result']

    # Sends response to a command
    def send_resp(self,packet):
        p_str=json.dumps(packet)+'\n'
        self.cs.send(p_str)

    # Sends an update packet
    def send_update(self, packet):
        p_str=json.dumps(packet)+'\n'
        self.us.send(p_str)
        # Right now, updates are not responded to
        #return self.recv_update_resp()

    # Gracefully close the open sockets
    def close(self):
        self.cs.close()
        self.us.close()

class JSONParentPoller(object):
    def __init__(self):
        self.connection = ParentConnection()
        # if modules need this then it's a personal problem
        super(JSONParentPoller,self).__init__()

    def close(self):
        return self.connection.close()

    def handle_one_command(self):
        data = self.connection.recv_cmd()
        try:
            if 'cmd' not in data:
                raise Exception("Malformed command")

            cmd=data['cmd']

            if cmd not in self.commands:
                print "Unrecognized command:", cmd, data
                raise Exception("Unrecognized command")

            cmd_f=self.commands[cmd]

            args = data.get("args", {})

            self.connection.send_resp(packet.good(cmd_f(self,**args)))

        except Exception as e:
            traceback.print_exc()
            self.connection.send_resp(packet.error(str(e)))

    def set_parameters(self, params):
        data = {"cmd": "set_parameters", "args": {"parameters": params}}
        return self.connection.send_update(data)

    def unset_parameters(self, params):
        data = {"cmd": "unset_parameters", "args": {"parameters": params}}
        return self.connection.send_update(data)

    def rm(self):
        data = {"cmd": "rm"}
        return self.connection.send_update(data)




import endpoints
import os
import werkzeug

not_found_app = werkzeug.exceptions.NotFound()

static_path = endpoints.static_endpoints["/"]

url_map = werkzeug.routing.Map([
    werkzeug.routing.Rule('/', endpoint='index.html'),
])

def application(environ, start_response):
    adapter = url_map.bind_to_environ(environ)
    try: 
        endpoint, values = adapter.match()
    except:
        return not_found_app(environ, start_response)
    else:
        file_path = os.path.join(static_path, endpoint)
        f = open(file_path)
        response = werkzeug.wrappers.Response(werkzeug.wsgi.wrap_file(environ, f), mimetype="text/html")
        return response(environ, start_response)


application = werkzeug.wsgi.SharedDataMiddleware(application, endpoints.static_endpoints)
application = werkzeug.wsgi.DispatcherMiddleware(application, endpoints.wsgi_endpoints)



import json
import os
import pkg_resources

json_path = os.environ.get("SHMOOZE_SETTINGS")
if json_path is None:
    json_path = pkg_resources.resource_filename("shmooze", '../settings.json')

print "Using shmooze settings file: '{}'".format(json_path)

settings_file=open(json_path)

settings = json.load(settings_file)

if 'static_path' not in settings:
    settings['static_path'] = pkg_resources.resource_filename("shmooze", '../static')

for key, value in settings.items():
    if key.endswith("_path"):
        settings[key] = os.path.expandvars(value)

globals().update(settings)



#!/usr/bin/env python

import fnmatch
import glob
import os
import sys

from setuptools import setup

with open("requirements.txt") as f:
    required = f.read().splitlines()

VERSION = "1.0.3"

setup(
    name='shmooze',
    version=VERSION,
    description='Framework for processed-backed web applications',
    author='Zach Banks',
    author_email='zbanks@mit.edu',
    url='https://github.com/zbanks/shmooze',
    packages=[
        'shmooze', 
        'shmooze.wsgi', 
        'shmooze.modules', 
        'shmooze.lib', 
    ],
    download_url="https://github.com/zbanks/shmooze/tarball/{}".format(VERSION),
    zip_safe=False,
    install_requires=required,
    scripts=[
        "bin/shmooze", 
        "bin/shmz", 
    ],
    package_dir = {
    },
    package_data={
        'musicazoo': [
            "../supervisord.conf", 
            "../requirements.txt", 
            "../settings.json",
            '../static/settings.json', 
            '../static/*.js', 
            '../static/*.html', 
        ],
    },
)



import json

class FileLogger(object):
    def __init__(self,filename):
        self.filename=filename

    def log(self,msg):
        f=open(self.filename,"a")
        with f:
            json.dump(msg,f)
            f.write('\n')



import shmooze.lib.service as service
import shmooze.lib.packet as packet
import socket
import tornado.iostream
import subprocess
import json
import datetime
import traceback
import uuid

# A module is an object on the queue.
# The actual code for a module runs in a sub-process.
# This class contains the infrastructure for starting, stopping, and communicating with that sub-process.

class Module(service.JSONCommandProcessor):
    # Hostname for listening socket
    # i.e. "Who is allowed to connect to the queue?"
    listen_host = 'localhost'
    # Hostname for connecting socket (passed to sub-process)
    # i.e. "Where does the queue process live?"
    connect_host = 'localhost'

    connect_timeout=datetime.timedelta(milliseconds=2000)
    cmd_write_timeout=datetime.timedelta(milliseconds=1000)
    cmd_read_timeout=datetime.timedelta(milliseconds=1000)
    natural_death_timeout=datetime.timedelta(milliseconds=3000) # give SIGTERM after 1 sec
    sigterm_timeout=datetime.timedelta(milliseconds=5000) # give SIGKILL after 1 sec

    # Make a new instance of this module.
    # This constructor is fairly bare because it is not a coroutine.
    # Most of the object initialization is done in new()
    def __init__(self,remove_function):
        self.parameters={}
        self.remove_function=remove_function
        self.cmd_lock = service.Lock()
        self.instance = str(uuid.uuid4())
        self.log_prefix = {"node":"module-queue","instance":self.instance}

    # Helper function for new()
    # Set up listening sockets for subprocess
    def listen(self):
        s1=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s1.bind((self.listen_host, 0))
        s1.listen(0)
        self.cmd_port = s1.getsockname()[1]
        s2=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s2.bind((self.listen_host, 0))
        s2.listen(0)
        self.update_port = s2.getsockname()[1]
        return [service.accept(s1),service.accept(s2)]

    # Helper function for new()
    # Launch subprocess
    def spawn(self):
        additional_args=[self.connect_host,str(self.cmd_port),str(self.update_port)]
        self.proc=subprocess.Popen(self.process+additional_args)
        self.alive=True

    # Helper function for new()
    # Set up IOstreams for the command and update connection objects
    def setup_connections(self,connections):
        conn1,conn2=tuple(connections)
        self.cmd_stream = tornado.iostream.IOStream(conn1[0])
        #self.cmd_stream.set_close_callback(self.on_disconnect)

        self.update_stream = tornado.iostream.IOStream(conn2[0])
        #self.update_stream.set_close_callback(self.on_disconnect)

    # Handles the majority of object initialization
    # Waits for socket communication to be established
    # and for the initialization command to return successfully
    @service.coroutine
    def new(self,args=None):
        # Set up two sockets for communication with the sub-process
        listen_futures = self.listen()
        # Launch the subprocess
        self.spawn()

        try:
            # Wait for the subprocess to connect
            try:
                connections = yield [service.with_timeout(self.connect_timeout,f) for f in listen_futures]
            except service.TimeoutError:
                raise Exception("Could not connect to spawned module")
            self.setup_connections(connections)

            # Helps the queue keep track of whether a module is playing or suspended
            self.is_on_top=False

            # Send initialization data to the sub-process
            try:
                result = yield self.send_cmd("init",args)
            except service.TimeoutError:
                raise Exception("Could not init spawned module")
        except Exception:
            self.terminate() # ensure process is dead if any sort of error occurs
            raise

        def poll_updates_done(f):
            if f.exception() is not None:
                traceback.print_exception(*f.exc_info())

        service.ioloop.add_future(self.poll_updates(),poll_updates_done) # Listen for updates from module forever

    # Called from queue
    # Stops the module, as it has been removed from the queue
    @service.coroutine
    def remove(self):
        yield self.send_cmd("rm")
        self.terminate()

    # Called from queue
    # Plays the module, as it has reached the top of the queue
    @service.coroutine
    def play(self):
        yield self.send_cmd("play")
        self.is_on_top=True

    # Called from queue
    # Suspends the module, as it has been bumped down from the top of the queue
    @service.coroutine
    def suspend(self):
        yield self.send_cmd("suspend")
        self.is_on_top=False

    # Called by queue
    # Retrieve some cached parameters
    def get_multiple_parameters(self,parameters):
        return dict([(p,self.parameters[p]) for p in parameters if p in self.parameters])

    # Called by queue
    # Issues a custom command to the module sub-process
    @service.coroutine
    def tell(self,cmd,args):
        result = yield self.send_cmd("do_"+cmd,args)
        raise service.Return(result)

    # Send a command to the sub-process over the command pipe
    @service.coroutine
    def send_cmd(self,cmd,args=None):
        cmd_dict={"cmd":cmd}
        if args is not None:
            cmd_dict["args"]=args
        cmd_str=json.dumps(cmd_dict)+'\n'

        toe=None
        # Lock on the command pipe so we ensure sequential req/rep transactions
        try:
            with (yield self.cmd_lock.acquire()):
                yield service.with_timeout(self.cmd_write_timeout,self.cmd_stream.write(cmd_str))
                response_str = yield service.with_timeout(self.cmd_read_timeout,self.cmd_stream.read_until('\n'))
        except (service.TimeoutError,tornado.iostream.StreamClosedError) as e:
            self.terminate()
            if self.logger is not None:
                self.logger.log(self.uid, "queue-module", cmd_dict, None)
            if isinstance(e,service.TimeoutError):
                raise Exception("Timeout sending message to module")
            if isinstance(e,tornado.iostream.StreamClosedError):
                raise Exception("Pipe to module unexpectedly closed")
            raise

        response_dict=json.loads(response_str)
        if self.logger is not None:
            self.logger.log(self.uid, "queue-module", cmd_dict, response_dict)
        raise service.Return(packet.assert_success(response_dict))

    # This function schedules a module's death and returns immediately.
    # It also removes the module from the queue if it is on it.
    def terminate(self):
        if self.alive:
            self.alive=False

            def remove_from_queue_done(f):
                pass

            def terminate_process_done(f):
                pass

            service.ioloop.add_future(self.remove_function(),remove_from_queue_done)
            service.ioloop.add_future(self.terminate_process(),terminate_process_done)

    # Ensure this module's sub-process is dead
    # Like, no really.
    @service.coroutine
    def terminate_process(self):
        try:
            self.cmd_stream.close()
            self.update_stream.close()
            yield service.with_timeout(self.natural_death_timeout,service.wait(self.proc))
        except (service.TimeoutError, AttributeError):
            print "Module was not dead, sending SIGTERM..."
            self.proc.terminate()
            try:
                yield service.with_timeout(self.sigterm_timeout,service.wait(self.proc))
            except service.TimeoutError:
                print "Module was not dead, sending SIGKILL..."
                self.proc.kill()
        except Exception:
            print "UNHANDLED EXCEPTION IN TERMINATE"
            traceback.print_exc()
            print "There is probably an orphaned child process!"

    # Poll for updates forever
    def poll_updates(self):
        return service.listen_for_commands(self.update_stream,self.command,self.terminate)

    @service.coroutine
    def set_parameters(self,parameters):
        self.parameters.update(parameters)

    @service.coroutine
    def unset_parameters(self,parameters):
        for parameter in parameters:
            self.parameters.pop(parameter, None)

    commands={
        'set_parameters':set_parameters,
        'unset_parameters':unset_parameters,
        'rm':service.coroutine(terminate),
    }

    log_cmds = ['set_parameters','unset_parameters','rm']




import json

def error(err):
    return {'success':False,'error':err}

def good(payload=None):
    if payload is not None:
        return {'success':True,'result':payload}
    return {'success':True}

def assert_success(response):
    if not isinstance(response,dict) or 'success' not in response:
        print "E",response
        raise Exception("Malformed response")
    if 'success' not in response:
        raise Exception("Malformed response")
    if response['success']:
        if 'result' in response:
            return response['result']
        return
    if 'error' not in response:
        raise Exception("Malformed response")
    raise Exception(response['error'])


